@@ -184,25 +184,25 @@ class ModelSelectionResults(collections.abc.Mapping):
     """
 
     def __init__(self, explored):
-        self._models = {key: res["model"] for key, res in explored}
-        self._bestid = max(explored.keys(), key=lambda x: explored[x]["icl"])
+        self._models = explored
+        self._best = max(explored.values(), key=lambda x: x.get_ICL())
 
     @property
     def best(self) -> Union[SBM, LBM]:
         """Model with higher ICL value."""
-        return self._models[self._bestid]
+        return self._best
 
     def __getitem__(self, key):
         return self._models[key]
 
     def keys(self):
-        return self._models.__keys__()
+        return self._models.keys()
 
     def values(self):
-        return self._models.__values__()
+        return self._models.values()
 
     def items(self):
-        return self._models.__items__()
+        return self._models.items()
 
     def __len__(self):
         return len(self._models)
