@@ -144,11 +144,11 @@ class ModelSelection:  # pylint: disable=too-many-instance-attributes
         return ModelSelectionResults(
             {
                 (
-                    model.n_row_clusters + model.n_column_clusters
+                    (model.n_row_clusters, model.n_column_clusters)
                     if self._model_type == "LBM"
                     else model.n_clusters
                 ): model
-                for model in self.model_explored.values()
+                for model in (expl["model"] for expl in self.model_explored.values())
             }
         )
 
