@@ -1,16 +1,46 @@
-import numpy as np
-import scipy
-from . import LBM, SBM
-from typing import Tuple, Union, Optional
-from scipy.sparse import coo_matrix
-from scipy.special import comb
+# Copyright 2020-2023, Université de technologie de Compiègne, France,
+#                      Gabriel Frisch <gabriel.frisch@hds.utc.fr>
+# Copyright 2022-2023, Université de technologie de Compiègne, France,
+#                      Jean-Benoist Leger <jbleger@hds.utc.fr>
+#
+# Permission is hereby granted, free of charge, to any person obtaining a
+# copy of this software and associated documentation files (the
+# "Software"), to deal in the Software without restriction, including
+# without limitation the rights to use, copy, modify, merge, publish,
+# distribute, sublicense, and/or sell copies of the Software, and to
+# permit persons to whom the Software is furnished to do so, subject to
+# the following conditions:
+#
+# The above copyright notice and this permission notice shall be included
+# in all copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
+# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
+# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
+# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+
+# pylint: disable=missing-module-docstring
+
 import logging
+from typing import Tuple, Union
+
+import scipy
+import numpy as np
+from scipy.special import comb
+from scipy.sparse import coo_matrix
+
+from . import LBM, SBM
 
 logger = logging.getLogger(__name__)
 
 
 def ARI(
-    labels_true: Union[np.ndarray, list], labels_pred: Union[np.ndarray, list]
+    # pylint: disable=unsubscriptable-object,invalid-name
+    labels_true: Union[np.ndarray, list],
+    labels_pred: Union[np.ndarray, list],
 ) -> float:
     """
     Adjusted Rand Index.
@@ -82,6 +112,7 @@ def ARI(
 
 
 def CARI(
+    # pylint: disable=unsubscriptable-object,too-many-locals,invalid-name
     labels_true_part_1: Union[np.ndarray, list],
     labels_true_part_2: Union[np.ndarray, list],
     labels_pred_part_1: Union[np.ndarray, list],
@@ -175,26 +206,14 @@ def CARI(
         return 1.0
 
     # Compute the contingency data tables
-    _, true_class_idx_part_1 = np.unique(
-        labels_true_part_1, return_inverse=True
-    )
-    _, pred_class_idx_part_1 = np.unique(
-        labels_pred_part_1, return_inverse=True
-    )
+    _, true_class_idx_part_1 = np.unique(labels_true_part_1, return_inverse=True)
+    _, pred_class_idx_part_1 = np.unique(labels_pred_part_1, return_inverse=True)
     contingency_part_1 = np.zeros((n_classes_part_1, n_clusters_part_1))
-    np.add.at(
-        contingency_part_1, (true_class_idx_part_1, pred_class_idx_part_1), 1
-    )
-    _, true_class_idx_part_2 = np.unique(
-        labels_true_part_2, return_inverse=True
-    )
-    _, pred_class_idx_part_2 = np.unique(
-        labels_pred_part_2, return_inverse=True
-    )
+    np.add.at(contingency_part_1, (true_class_idx_part_1, pred_class_idx_part_1), 1)
+    _, true_class_idx_part_2 = np.unique(labels_true_part_2, return_inverse=True)
+    _, pred_class_idx_part_2 = np.unique(labels_pred_part_2, return_inverse=True)
     contingency_part_2 = np.zeros((n_classes_part_2, n_clusters_part_2))
-    np.add.at(
-        contingency_part_2, (true_class_idx_part_2, pred_class_idx_part_2), 1
-    )
+    np.add.at(contingency_part_2, (true_class_idx_part_2, pred_class_idx_part_2), 1)
 
     # Theorem 3.3 of Robert2019 (https://hal.inria.fr/hal-01524832v4) defines
     # the final contingency matrix by the Kronecker product between the two
@@ -212,13 +231,14 @@ def CARI(
 
 
 def lbm_merge_group(
+    # pylint: disable=invalid-name,redefined-builtin,too-many-locals,missing-return-doc,missing-return-type-doc
     model: LBM,
     type: int,
     idx_group_1: int,
     idx_group_2: int,
     indices_ones: np.ndarray,
 ) -> Tuple[float, LBM]:
-    """ Given a LBM model, returns the model obtained from the merge of the specified classes.
+    """Given a LBM model, returns the model obtained from the merge of the specified classes.
 
     Parameters
     ----------
@@ -232,15 +252,20 @@ def lbm_merge_group(
         0 for rows merging, 1 for columns merging.
     indices_ones : numpy.ndarray
         Indices of elements that are non-zero in the original data matrix.
+
     Returns
     -------
     tuple of (float, sparsebm.LBM)
         The ICL value and the model obtained from the merge of two classes.
+
+    Raises
+    ------
+    ValueError
+        if type is neither 0 or 1.
     """
-    if type != 0 and type != 1:
+    if type not in (0, 1):
         logger.error("Type error in merge group")
-        assert False
-    eps = 1e-4
+        raise ValueError("type must be 0 or 1")
     if type == 0:
         model.n_row_clusters -= 1
         t = model.tau_1_
@@ -250,10 +275,7 @@ def lbm_merge_group(
         t = model.tau_2_
         alpha = model.alpha_2_
     if idx_group_1 > idx_group_2:
-        c = idx_group_2
-        idx_group_2 = idx_group_1
-        idx_group_1 = c
-    n = t.shape[0]
+        idx_group_1, idx_group_2 = idx_group_2, idx_group_1
 
     new_t = np.delete(t, idx_group_2, axis=1)
     new_t[:, idx_group_1] = t[:, idx_group_1] + t[:, idx_group_2]
@@ -278,9 +300,8 @@ def lbm_merge_group(
         ) / (alpha[idx_group_1] + alpha[idx_group_2])
 
     model.pi_ = new_pi
-    nq = model.n_row_clusters
-    nl = model.n_column_clusters
 
+    # pylint: disable=protected-access
     # Transfert to GPU if necessary
     t1 = model._np.asarray(model.tau_1_)
     t2 = model._np.asarray(model.tau_2_)
@@ -293,9 +314,13 @@ def lbm_merge_group(
 
 
 def sbm_merge_group(
-    model: SBM, idx_group_1: int, idx_group_2: int, indices_ones: np.ndarray
+    # pylint: disable=too-many-locals,missing-return-doc
+    model: SBM,
+    idx_group_1: int,
+    idx_group_2: int,
+    indices_ones: np.ndarray,
 ) -> Tuple[float, SBM]:
-    """ Given a SBM model, returns the model obtained from the merge of the specified classes.
+    """Given a SBM model, returns the model obtained from the merge of the specified classes.
 
     Parameters
     ----------
@@ -307,21 +332,19 @@ def sbm_merge_group(
         index of the second row/column class to merge.
     indices_ones : numpy.ndarray
         Indices of elements that are non-zero in the original data matrix.
+
     Returns
     -------
     tuple of (float, sparsebm.SBM)
         The ICL value and the model obtained from the merge of two classes.
     """
-    eps = 1e-4
+    # pylint: disable=invalid-name
     model.n_clusters -= 1
     t = model.tau_
     alpha = model.alpha_
 
     if idx_group_1 > idx_group_2:
-        c = idx_group_2
-        idx_group_2 = idx_group_1
-        idx_group_1 = c
-    n = t.shape[0]
+        idx_group_1, idx_group_2 = idx_group_2, idx_group_1
 
     new_t = np.delete(t, idx_group_2, axis=1)
     new_t[:, idx_group_1] = t[:, idx_group_1] + t[:, idx_group_2]
@@ -332,13 +355,13 @@ def sbm_merge_group(
     model.tau_ = new_t
     nq = model.n_clusters
 
+    # pylint: disable=protected-access
     # Transfert to GPU if necessary
     t1 = model._np.asarray(model.tau_)
     a1 = model._np.asarray(model.alpha_)
     t1_sum = t1.sum(0)
     pi = (
-        t1[indices_ones[0]].reshape(-1, nq, 1)
-        * t1[indices_ones[1]].reshape(-1, 1, nq)
+        t1[indices_ones[0]].reshape(-1, nq, 1) * t1[indices_ones[1]].reshape(-1, 1, nq)
     ).sum(0) / ((t1_sum.reshape((-1, 1)) * t1_sum) - t1.T @ t1)
 
     ll = model._compute_likelihood(indices_ones, pi, a1, t1)
@@ -348,13 +371,14 @@ def sbm_merge_group(
 
 
 def lbm_split_group(
+    # pylint: disable=missing-return-doc,missing-return-type-doc,too-many-locals,redefined-builtin
     model: LBM,
     row_col_degrees: Tuple[np.ndarray],
     type: int,
     index: int,
     indices_ones: np.array,
 ) -> Tuple[float, LBM]:
-    """ Given a LBM model, returns the model obtained from the split of the specified class.
+    """Given a LBM model, returns the model obtained from the split of the specified class.
 
     The specified class is splitted according to its median of degree.
 
@@ -371,14 +395,21 @@ def lbm_split_group(
         index of the row/column class to split.
     indices_ones : numpy.ndarray
         Indices of elements that are non-zero in the original data matrix.
+
     Returns
     -------
     tuple of (float, sparsebm.LBM)
         The ICL value and the model obtained from the split of the specified class.
+
+    Raises
+    ------
+    ValueError
+        When type is neighter 0 or 1.
     """
-    if type != 0 and type != 1:
+    # pylint: disable=invalid-name
+    if type not in (0, 1):
         logger.error("Type error in split group")
-        assert False
+        raise ValueError("Type must be 0 or 1.")
     eps = 1e-4
     if type == 0:
         model.n_row_clusters += 1
@@ -408,6 +439,7 @@ def lbm_split_group(
     nq = model.n_row_clusters
     nl = model.n_column_clusters
 
+    # pylint: disable=protected-access
     # Transfert to GPU if necessary
     t1 = model._np.asarray(model.tau_1_)
     t2 = model._np.asarray(model.tau_2_)
@@ -415,8 +447,7 @@ def lbm_split_group(
     a2 = model._np.asarray(model.alpha_2_)
 
     pi = (
-        t1[indices_ones[0]].reshape(-1, nq, 1)
-        * t2[indices_ones[1]].reshape(-1, 1, nl)
+        t1[indices_ones[0]].reshape(-1, nq, 1) * t2[indices_ones[1]].reshape(-1, 1, nl)
     ).sum(0) / (t1.sum(0).reshape(nq, 1) * t2.sum(0).reshape(1, nl))
 
     model.pi_ = pi.get() if model.use_gpu else pi
@@ -427,9 +458,13 @@ def lbm_split_group(
 
 
 def sbm_split_group(
-    model: SBM, degrees: np.ndarray, index: int, indices_ones: np.array
+    # pylint: disable=missing-return-doc,missing-return-type-doc,too-many-locals,redefined-builtin
+    model: SBM,
+    degrees: np.ndarray,
+    index: int,
+    indices_ones: np.array,
 ):
-    """ Given a SBM model, returns the model obtained from the split of the specified class.
+    """Given a SBM model, returns the model obtained from the split of the specified class.
 
     The specified class is splitted according to its median of degree.
 
@@ -448,6 +483,7 @@ def sbm_split_group(
     tuple of (float, sparsebm.SBM)
         The ICL value and the model obtained from the split of the specified class.
     """
+    # pylint: disable=invalid-name
     eps = 1e-4
     model.n_clusters += 1
     t = model.tau_
@@ -467,14 +503,14 @@ def sbm_split_group(
     model.alpha_ = t.mean(0)
     nq = model.n_clusters
 
+    # pylint: disable=protected-access
     # Transfert to GPU if necessary
     t1 = model._np.asarray(model.tau_)
     a1 = model._np.asarray(model.alpha_)
     t1_sum = t1.sum(0)
 
     pi = (
-        t1[indices_ones[0]].reshape(-1, nq, 1)
-        * t1[indices_ones[1]].reshape(-1, 1, nq)
+        t1[indices_ones[0]].reshape(-1, nq, 1) * t1[indices_ones[1]].reshape(-1, 1, nq)
     ).sum(0) / ((t1_sum.reshape((-1, 1)) * t1_sum) - t1.T @ t1)
 
     model.pi_ = pi.get() if model.use_gpu else pi
@@ -484,8 +520,10 @@ def sbm_split_group(
     return (model.get_ICL(), model)
 
 
-def reorder_rows(X: coo_matrix, idx: np.ndarray) -> None:
-    """ Reorders the rows of the COO sparse matrix given in argument.
+def reorder_rows(  # pylint: disable=invalid-name,missing-param-doc
+    X: coo_matrix, idx: np.ndarray
+) -> None:
+    """Reorders the rows of the COO sparse matrix given in argument.
 
     Parameters
     ----------
@@ -495,10 +533,45 @@ def reorder_rows(X: coo_matrix, idx: np.ndarray) -> None:
         Row indices used to reorder the matrix.
     """
     idx = idx.flatten()
-    assert isinstance(
-        X, scipy.sparse.coo_matrix
-    ), "X must be scipy.sparse.coo_matrix"
+    assert isinstance(X, scipy.sparse.coo_matrix), "X must be scipy.sparse.coo_matrix"
     assert X.shape[0] == idx.shape[0], "idx shape[0] must be X shape[0]"
     idx = np.argsort(idx)
     idx = np.asarray(idx, dtype=X.row.dtype)
     X.row = idx[X.row]
+
+
+def reorder_cols(  # pylint: disable=invalid-name,missing-param-doc
+    X: coo_matrix, idx: np.ndarray
+) -> None:
+    """Reorders the columns of the COO sparse matrix given in argument.
+
+    Parameters
+    ----------
+    X : scipy.sparse.coo_matrix
+        The sparse matrix to reorder.
+    idx: numpy.ndarray,  shape=(X.shape[1],)
+        Row indices used to reorder the matrix.
+    """
+    idx = idx.flatten()
+    assert isinstance(X, scipy.sparse.coo_matrix), "X must be scipy.sparse.coo_matrix"
+    assert X.shape[1] == idx.shape[0], "idx shape[0] must be X shape[0]"
+    idx = np.argsort(idx)
+    idx = np.asarray(idx, dtype=X.col.dtype)
+    X.col = idx[X.col]
+
+
+def reorder_rows_and_cols(  # pylint: disable=invalid-name,missing-param-doc
+    X: coo_matrix, idx: np.ndarray
+) -> None:
+    """Reorders the columns of the COO sparse matrix given in argument.
+
+    Parameters
+    ----------
+    X : scipy.sparse.coo_matrix
+        The sparse matrix to reorder. Must be square matrix.
+    idx: numpy.ndarray,  shape=(X.shape[0],)
+        Row indices used to reorder the matrix.
+    """
+
+    reorder_rows(X, idx)
+    reorder_cols(X, idx)
